@@ -1,4 +1,4 @@
-#![cfg_attr(not(test), no_std)]
+#![no_std]
 #![forbid(missing_docs)]
 
 //! A library for defining enums that can be used in compact bit sets. It supports enums up to 128
@@ -32,7 +32,7 @@
 //!
 //! All bitwise operations you would expect to work on bitsets also work on both EnumSets and
 //! enums with `#[derive(EnumSetType)]`:
-//! ```
+//! ```rust
 //! # use enumset::*;
 //! # #[derive(EnumSetType, Debug)] pub enum Enum { A, B, C, D, E, F, G }
 //! // Intersection of sets
@@ -73,17 +73,12 @@
 //! assert_eq!(set, Enum::A | Enum::E | Enum::G);
 //! ```
 
-#[cfg(test)] extern crate core;
-extern crate enumset_derive;
-extern crate num_traits;
-#[cfg(feature = "serde")] extern crate serde2 as serde;
-
 pub use enumset_derive::*;
-mod enumset { pub use super::*; }
 
+use core::cmp::Ordering;
 use core::fmt;
 use core::fmt::{Debug, Formatter};
-use core::hash::Hash;
+use core::hash::{Hash, Hasher};
 use core::ops::*;
 
 use num_traits::*;
@@ -100,11 +95,29 @@ pub mod internal {
     }
 
     /// A reexport of core to allow our macros to be generic to std vs core.
-    pub extern crate core;
+    pub use ::core as core_export;
 
     /// A reexport of serde so there is no requirement to depend on serde.
-    #[cfg(feature = "serde")] pub extern crate serde2 as serde;
+    #[cfg(feature = "serde")] pub use serde2 as serde;
+
+    /// The actual members of EnumSetType. Put here to avoid polluting global namespaces.
+    pub unsafe trait EnumSetTypePrivate {
+        type Repr: EnumSetTypeRepr;
+        const ALL_BITS: Self::Repr;
+        fn enum_into_u8(self) -> u8;
+        unsafe fn enum_from_u8(val: u8) -> Self;
+
+        #[cfg(feature = "serde")]
+        fn serialize<S: serde::Serializer>(set: EnumSet<Self>, ser: S) -> Result<S::Ok, S::Error>
+            where Self: EnumSetType;
+        #[cfg(feature = "serde")]
+        fn deserialize<'de, D: serde::Deserializer<'de>>(de: D) -> Result<EnumSet<Self>, D::Error>
+            where Self: EnumSetType;
+    }
 }
+use crate::internal::EnumSetTypePrivate;
+#[cfg(feature = "serde")] use crate::internal::serde;
+#[cfg(feature = "serde")] use crate::serde::{Serialize, Deserialize};
 
 mod private {
     use super::*;
@@ -124,7 +137,7 @@ mod private {
     prim!(u64 , 64 );
     prim!(u128, 128);
 }
-use private::EnumSetTypeRepr;
+use crate::private::EnumSetTypeRepr;
 
 /// The trait used to define enum types that may be used with [`EnumSet`].
 ///
@@ -133,14 +146,13 @@ use private::EnumSetTypeRepr;
 ///
 /// # Custom Derive
 ///
-/// The custom derive for `EnumSetType` automatically creates implementations of [`PartialEq`],
+/// The custom derive for [`EnumSetType`] automatically creates implementations of [`PartialEq`],
 /// [`Sub`], [`BitAnd`], [`BitOr`], [`BitXor`], and [`Not`] allowing the enum to be used as
-/// if it were an [`EnumSet`] in expressions. This can be disabled by adding an `#[enumset_no_ops]`
+/// if it were an [`EnumSet`] in expressions. This can be disabled by adding an `#[enumset(no_ops)]`
 /// annotation to the enum.
 ///
-/// The custom derive for `EnumSetType` also automatically creates implementations equivalent to
-/// `#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash)]`. This can be disabled by adding
-/// an `#[enumset_no_derives]` annotation to the enum.
+/// The custom derive for `EnumSetType` automatically implements [`Copy`], [`Clone`], [`Eq`], and
+/// [`PartialEq`] on the enum. These are required for the [`EnumSet`] to function.
 ///
 /// Any C-like enum is supported, as long as there are no more than 128 variants in the enum,
 /// and no variant discriminator is larger than 127.
@@ -151,7 +163,7 @@ use private::EnumSetTypeRepr;
 ///
 /// ```rust
 /// # use enumset::*;
-/// #[derive(EnumSetType, Debug)]
+/// #[derive(EnumSetType)]
 /// pub enum Enum {
 ///    A, B, C, D, E, F, G,
 /// }
@@ -161,7 +173,7 @@ use private::EnumSetTypeRepr;
 ///
 /// ```rust
 /// # use enumset::*;
-/// #[derive(EnumSetType, Debug)]
+/// #[derive(EnumSetType)]
 /// pub enum SparseEnum {
 ///    A = 10, B = 20, C = 30, D = 127,
 /// }
@@ -171,26 +183,32 @@ use private::EnumSetTypeRepr;
 ///
 /// ```rust
 /// # use enumset::*;
-/// #[derive(EnumSetType, Debug)]
-/// #[enumset_no_ops]
+/// #[derive(EnumSetType)]
+/// #[enumset(no_ops)]
 /// pub enum NoOpsEnum {
 ///    A, B, C, D, E, F, G,
 /// }
 /// ```
-pub unsafe trait EnumSetType: Copy {
-    #[doc(hidden)] type Repr: EnumSetTypeRepr;
-    #[doc(hidden)] const ALL_BITS: Self::Repr;
-    #[doc(hidden)] fn enum_into_u8(self) -> u8;
-    #[doc(hidden)] unsafe fn enum_from_u8(val: u8) -> Self;
-
-    #[cfg(feature = "serde")] #[doc(hidden)]
-    fn serialize<S: serde::Serializer>(set: EnumSet<Self>, ser: S) -> Result<S::Ok, S::Error>;
-    #[cfg(feature = "serde")] #[doc(hidden)]
-    fn deserialize<'de, D: serde::Deserializer<'de>>(de: D) -> Result<EnumSet<Self>, D::Error>;
-}
+pub unsafe trait EnumSetType: Copy + Eq + EnumSetTypePrivate { }
 
 /// An efficient set type for enums.
-#[derive(Copy, Clone, PartialOrd, Ord, PartialEq, Eq, Hash)]
+///
+/// # Serialization
+///
+/// The default representation serializes enumsets as an `u8`, `u16`, `u32`, `u64`, or `u128`,
+/// whichever is the smallest that can contain all bits that are part of the set.
+///
+/// Unknown bits are ignored, and are simply dropped. To override this behavior, you can add a
+/// `#[enumset(serialize_deny_unknown)]` annotation to your enum.
+///
+/// You can add a `#[enumset(serialize_repr = "u8")]` annotation to your enum to explicitly set
+/// the bit width the `EnumSet` is serialized as. This can be used to avoid breaking changes in
+/// certain serialization formats (such as `bincode`).
+///
+/// In addition, the `#[enumset(serialize_as_list)]` annotation causes the `EnumSet` to be
+/// instead serialized as a list. This requires your enum type implement [`Serialize`] and
+/// [`Deserialize`].
+#[derive(Copy, Clone, PartialEq, Eq)]
 pub struct EnumSet<T : EnumSetType> {
     #[doc(hidden)]
     /// This is public due to the [`enum_set!`] macro.
@@ -431,7 +449,7 @@ impl <T : EnumSetType> PartialEq<T> for EnumSet<T> {
     }
 }
 impl <T : EnumSetType + Debug> Debug for EnumSet<T> {
-    fn fmt(&self, f: &mut Formatter) -> fmt::Result {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         let mut is_first = true;
         f.write_str("EnumSet(")?;
         for v in self.iter() {
@@ -444,15 +462,31 @@ impl <T : EnumSetType + Debug> Debug for EnumSet<T> {
     }
 }
 
+impl <T: EnumSetType> Hash for EnumSet<T> {
+    fn hash<H: Hasher>(&self, state: &mut H) {
+        self.__enumset_underlying.hash(state)
+    }
+}
+impl <T: EnumSetType> PartialOrd for EnumSet<T> {
+    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+        self.__enumset_underlying.partial_cmp(&other.__enumset_underlying)
+    }
+}
+impl <T: EnumSetType> Ord for EnumSet<T> {
+    fn cmp(&self, other: &Self) -> Ordering {
+        self.__enumset_underlying.cmp(&other.__enumset_underlying)
+    }
+}
+
 #[cfg(feature = "serde")]
-impl <T : EnumSetType> serde::Serialize for EnumSet<T> {
+impl <T : EnumSetType> Serialize for EnumSet<T> {
     fn serialize<S: serde::Serializer>(&self, serializer: S) -> Result<S::Ok, S::Error> {
         T::serialize(*self, serializer)
     }
 }
 
 #[cfg(feature = "serde")]
-impl <'de, T : EnumSetType> serde::Deserialize<'de> for EnumSet<T> {
+impl <'de, T : EnumSetType> Deserialize<'de> for EnumSet<T> {
     fn deserialize<D: serde::Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
         T::deserialize(deserializer)
     }
@@ -481,38 +515,11 @@ impl <T : EnumSetType> Iterator for EnumSetIter<T> {
     }
 }
 
-/// Defines enums which can be used with EnumSet.
-///
-/// [`Copy`], [`Clone`], [`PartialOrd`], [`Ord`], [`PartialEq`], [`Eq`], [`Hash`], [`Debug`],
-/// [`Sub`], [`BitAnd`], [`BitOr`], [`BitXor`], and [`Not`] are automatically derived for the enum.
-///
-/// These impls, in general, behave as if the enum variant was an [`EnumSet`] with a single value,
-/// as those created by [`EnumSet::only`].
-#[macro_export]
-#[deprecated(since = "0.3.13", note = "Use `#[derive(EnumSetType)] instead.")]
-macro_rules! enum_set_type {
-    ($(#[$enum_attr:meta])* $vis:vis enum $enum_name:ident {
-        $($(#[$attr:meta])* $variant:ident),* $(,)*
-    } $($rest:tt)*) => {
-        $(#[$enum_attr])* #[repr(u8)]
-        #[derive($crate::EnumSetType, Debug)]
-        $vis enum $enum_name {
-            $($(#[$attr])* $variant,)*
-        }
-
-        enum_set_type!($($rest)*);
-    };
-    () => { };
-}
-
 /// Creates a EnumSet literal, which can be used in const contexts.
 ///
 /// The syntax used is `enum_set!(Type::A | Type::B | Type::C)`. Each variant must be of the same
 /// type, or a error will occur at compile-time.
 ///
-/// You may also explicitly state the type of the variants that follow, as in
-/// `enum_set!(Type, Type::A | Type::B | Type::C)`.
-///
 /// # Examples
 ///
 /// ```rust
@@ -520,9 +527,6 @@ macro_rules! enum_set_type {
 /// # #[derive(EnumSetType, Debug)] enum Enum { A, B, C }
 /// const CONST_SET: EnumSet<Enum> = enum_set!(Enum::A | Enum::B);
 /// assert_eq!(CONST_SET, Enum::A | Enum::B);
-///
-/// const EXPLICIT_CONST_SET: EnumSet<Enum> = enum_set!(Enum, Enum::A | Enum::B);
-/// assert_eq!(EXPLICIT_CONST_SET, Enum::A | Enum::B);
 /// ```
 ///
 /// This macro is strongly typed. For example, the following will not compile:
@@ -546,229 +550,4 @@ macro_rules! enum_set {
             },
         }.enum_set
     };
-    ($enum_name:ty, $($value:path)|* $(|)*) => {
-        $crate::EnumSet::<$enum_name> {
-            __enumset_underlying: 0 $(| (1 << ($value as $enum_name as u8)))*
-        }
-    }
-}
-
-#[cfg(test)]
-#[allow(dead_code)]
-mod test {
-    use super::*;
-
-    #[cfg(feature = "serde")]
-    extern crate bincode;
-
-    mod enums {
-        #[derive(::EnumSetType, Debug)]
-        pub enum SmallEnum {
-            A, B, C, D, E, F, G, H, I, J, K, L, M, N, O, P, Q, R, S, T, U, V, W, X, Y, Z,
-        }
-        #[derive(::EnumSetType, Debug)]
-        pub enum LargeEnum {
-            _00,  _01,  _02,  _03,  _04,  _05,  _06,  _07,
-            _10,  _11,  _12,  _13,  _14,  _15,  _16,  _17,
-            _20,  _21,  _22,  _23,  _24,  _25,  _26,  _27,
-            _30,  _31,  _32,  _33,  _34,  _35,  _36,  _37,
-            _40,  _41,  _42,  _43,  _44,  _45,  _46,  _47,
-            _50,  _51,  _52,  _53,  _54,  _55,  _56,  _57,
-            _60,  _61,  _62,  _63,  _64,  _65,  _66,  _67,
-            _70,  _71,  _72,  _73,  _74,  _75,  _76,  _77,
-            A, B, C, D, E, F, G, H, I, J, K, L, M, N, O, P, Q, R, S, T, U, V, W, X, Y, Z,
-        }
-        #[derive(::EnumSetType, Debug)]
-        pub enum Enum8 {
-            A, B, C, D, E, F, G, H,
-        }
-        #[derive(::EnumSetType, Debug)]
-        pub enum Enum128 {
-            A, B, C, D, E, F, G, H, _8, _9, _10, _11, _12, _13, _14, _15,
-            _16, _17, _18, _19, _20, _21, _22, _23, _24, _25, _26, _27, _28, _29, _30, _31,
-            _32, _33, _34, _35, _36, _37, _38, _39, _40, _41, _42, _43, _44, _45, _46, _47,
-            _48, _49, _50, _51, _52, _53, _54, _55, _56, _57, _58, _59, _60, _61, _62, _63,
-            _64, _65, _66, _67, _68, _69, _70, _71, _72, _73, _74, _75, _76, _77, _78, _79,
-            _80, _81, _82, _83, _84, _85, _86, _87, _88, _89, _90, _91, _92, _93, _94, _95,
-            _96, _97, _98, _99, _100, _101, _102, _103, _104, _105, _106, _107, _108, _109,
-            _110, _111, _112, _113, _114, _115, _116, _117, _118, _119, _120, _121, _122,
-            _123, _124,  _125, _126, _127,
-        }
-        #[derive(::EnumSetType, Debug)]
-        pub enum SparseEnum {
-            A = 10, B = 20, C = 30, D = 40, E = 50, F = 60, G = 70, H = 80,
-        }
-    }
-    use self::enums::*;
-
-    macro_rules! test_variants {
-        ($enum_name:ident $all_empty_test:ident $($variant:ident,)*) => {
-            #[test]
-            fn $all_empty_test() {
-                let all = EnumSet::<$enum_name>::all();
-                let empty = EnumSet::<$enum_name>::empty();
-
-                $(
-                    assert!(!empty.contains($enum_name::$variant));
-                    assert!(all.contains($enum_name::$variant));
-                )*
-            }
-        }
-    }
-    test_variants! { SmallEnum small_enum_all_empty
-        A, B, C, D, E, F, G, H, I, J, K, L, M, N, O, P, Q, R, S, T, U, V, W, X, Y, Z,
-    }
-    test_variants! { LargeEnum large_enum_all_empty
-        _00,  _01,  _02,  _03,  _04,  _05,  _06,  _07,
-        _10,  _11,  _12,  _13,  _14,  _15,  _16,  _17,
-        _20,  _21,  _22,  _23,  _24,  _25,  _26,  _27,
-        _30,  _31,  _32,  _33,  _34,  _35,  _36,  _37,
-        _40,  _41,  _42,  _43,  _44,  _45,  _46,  _47,
-        _50,  _51,  _52,  _53,  _54,  _55,  _56,  _57,
-        _60,  _61,  _62,  _63,  _64,  _65,  _66,  _67,
-        _70,  _71,  _72,  _73,  _74,  _75,  _76,  _77,
-        A, B, C, D, E, F, G, H, I, J, K, L, M, N, O, P, Q, R, S, T, U, V, W, X, Y, Z,
-    }
-    test_variants! { SparseEnum sparse_enum_all_empty
-        A, B, C, D, E, F, G,
-    }
-
-    macro_rules! test_enum {
-        ($e:ident, $m:ident) => {
-            mod $m {
-                use super::*;
-
-                const CONST_SET: EnumSet<$e> = enum_set!($e, $e::A | $e::C);
-                const EMPTY_SET: EnumSet<$e> = enum_set!();
-                #[test]
-                fn const_set() {
-                    assert_eq!(CONST_SET.len(), 2);
-                    assert!(CONST_SET.contains($e::A));
-                    assert!(CONST_SET.contains($e::C));
-                    assert!(EMPTY_SET.is_empty());
-                }
-
-                #[test]
-                fn basic_add_remove() {
-                    let mut set = EnumSet::new();
-                    set.insert($e::A);
-                    set.insert($e::B);
-                    set.insert($e::C);
-                    assert_eq!(set, $e::A | $e::B | $e::C);
-                    set.remove($e::B);
-                    assert_eq!(set, $e::A | $e::C);
-                    set.insert($e::D);
-                    assert_eq!(set, $e::A | $e::C | $e::D);
-                    set.insert_all($e::F | $e::E | $e::G);
-                    assert_eq!(set, $e::A | $e::C | $e::D | $e::F | $e::E | $e::G);
-                    set.remove_all($e::A | $e::D | $e::G);
-                    assert_eq!(set, $e::C | $e::F | $e::E);
-                    assert!(!set.is_empty());
-                    set.clear();
-                    assert!(set.is_empty());
-                }
-
-                #[test]
-                fn empty_is_empty() {
-                    assert_eq!(EnumSet::<$e>::empty().len(), 0)
-                }
-
-                #[test]
-                fn all_len() {
-                    assert_eq!(EnumSet::<$e>::all().len(), EnumSet::<$e>::variant_count() as usize)
-                }
-
-                #[test]
-                fn basic_iter_test() {
-                    let mut set = EnumSet::new();
-                    set.insert($e::A);
-                    set.insert($e::B);
-                    set.insert($e::C);
-                    set.insert($e::E);
-
-                    let mut set_2 = EnumSet::new();
-                    let vec: Vec<$e> = set.iter().collect();
-                    for val in vec {
-                        assert!(!set_2.contains(val));
-                        set_2.insert(val);
-                    }
-                    assert_eq!(set, set_2);
-
-                    let mut set_3 = EnumSet::new();
-                    for val in set {
-                        assert!(!set_3.contains(val));
-                        set_3.insert(val);
-                    }
-                    assert_eq!(set, set_3);
-                }
-
-                #[test]
-                fn basic_ops_test() {
-                    assert_eq!(($e::A | $e::B) | ($e::B | $e::C), $e::A | $e::B | $e::C);
-                    assert_eq!(($e::A | $e::B) & ($e::B | $e::C), $e::B);
-                    assert_eq!(($e::A | $e::B) ^ ($e::B | $e::C), $e::A | $e::C);
-                    assert_eq!(($e::A | $e::B) - ($e::B | $e::C), $e::A);
-                }
-
-                #[test]
-                fn basic_set_status() {
-                    assert!(($e::A | $e::B | $e::C).is_disjoint($e::D | $e::E | $e::F));
-                    assert!(!($e::A | $e::B | $e::C | $e::D).is_disjoint($e::D | $e::E | $e::F));
-                    assert!(($e::A | $e::B).is_subset($e::A | $e::B | $e::C));
-                    assert!(!($e::A | $e::D).is_subset($e::A | $e::B | $e::C));
-                }
-
-                #[test]
-                fn debug_impl() {
-                    assert_eq!(format!("{:?}", $e::A | $e::B | $e::D), "EnumSet(A | B | D)");
-                }
-
-                #[test]
-                fn to_from_bits() {
-                    let value = $e::A | $e::C | $e::D | $e::F | $e::E | $e::G;
-                    assert_eq!(EnumSet::from_bits(value.to_bits()), value);
-                }
-
-                #[test]
-                #[should_panic]
-                fn too_many_bits() {
-                    if EnumSet::<$e>::variant_count() == 128 {
-                        panic!("(test skipped)")
-                    }
-                    EnumSet::<$e>::from_bits(!0);
-                }
-
-                #[test]
-                fn match_const_test() {
-                    match CONST_SET {
-                        CONST_SET => { /* ok */ }
-                        _ => panic!("match fell through?"),
-                    }
-                }
-
-                #[test]
-                #[cfg(feature = "serde")]
-                fn serialize_deserialize_test() {
-                    let value = $e::A | $e::C | $e::D | $e::F | $e::E | $e::G;
-                    let serialized = bincode::serialize(&value).unwrap();
-                    let deserialized = bincode::deserialize::<EnumSet<$e>>(&serialized).unwrap();
-                    assert_eq!(value, deserialized);
-                }
-
-                #[test]
-                #[cfg(feature = "serde")]
-                fn deserialize_all_test() {
-                    let serialized = bincode::serialize(&(!0 as <$e as EnumSetType>::Repr)).unwrap();
-                    let deserialized = bincode::deserialize::<EnumSet<$e>>(&serialized).unwrap();
-                    assert_eq!(EnumSet::<$e>::all(), deserialized);
-                }
-            }
-        }
-    }
-
-    test_enum!(SmallEnum, small_enum);
-    test_enum!(LargeEnum, large_enum);
-    test_enum!(Enum8, enum8);
-    test_enum!(Enum128, enum128);
-    test_enum!(SparseEnum, sparse_enum);
 }
